bool g_inCustomMenu = false;

bool canUseLocalPlay = true;
bool canUseEditors = true;

bool enabledRequested = false;
bool disabledRequested = false;
bool updateRequested = false;
bool downloadAllMapsRequested = false;
bool isUpdating = false;
bool isEnabled = false;

int[] removedLayerIds;

void RenderMenu()
{
	if (UI::BeginMenu("\\$9cf" + Icons::LevelUp + "\\$z Platform titlepack")) {
		if (UI::MenuItem("Enter titlepack", "", false, g_inCustomMenu && canUseLocalPlay && !isEnabled && !isUpdating)) {
			EnterTitlepack();
		}
		if (UI::MenuItem("Update files", "", false, !isEnabled && !isUpdating)) {	
			UpdateTitlepack();
		}
		if (UI::MenuItem("Leave titlepack", "", false, isEnabled && !isUpdating)) {	
			LeaveTitlepack();
		}

		UI::EndMenu();
	}
}

void ShowCheckForUpdateUI() {
	if(TitlepackUpdater::IsUpToDate())
		UI::ShowNotification("\\$0d0" + Icons::Info + "Platform titlepack is up to date!", "", 5000);
	else
		UI::ShowNotification("\\$ff5" + Icons::Info + "Platform titlepack - update available!", "You'll be propmpted to update once you enter the titlepack.", 5000);
}

void EnterTitlepack() {
	auto app = cast<CGameManiaPlanet>(GetApp());
	app.LoadedManiaTitle.IdName = "TMNextPlatform@Arkady_TM";
	enabledRequested = true;
}

void UpdateTitlepack() {
	updateRequested = true;
}

void DownloadAllMaps() {
	downloadAllMapsRequested = true;
}

void Game_RefreshMapList() {
	auto app = GetTmApp();
	auto dataFileMgr = app.MenuManager.MenuCustom_CurrentManiaApp.DataFileMgr;
	dataFileMgr.Map_RefreshFromDisk();
}

void LeaveTitlepack() {
	auto app = cast<CGameManiaPlanet>(GetApp());
	app.LoadedManiaTitle.IdName = "Trackmania";
	disabledRequested = true;
	MLHook::RemoveAllInjectedML();
}

void ShowUpdateDialog() {
	MLHook::RemoveAllInjectedML();
	CGameUILayer@ layer = GetTitlepackMenuLayer();
	layer.IsVisible = true;
	layer.ManialinkPage = GetUpdateDialog();
}

void ShowDemoDialog() {
	MLHook::RemoveAllInjectedML();
	CGameUILayer@ layer = GetTitlepackMenuLayer();
	layer.IsVisible = true;
	layer.ManialinkPage = GetDemoInfoDialog();
}

void HideAllDialogs() {
	MLHook::InjectManialinkToMenu("TitlepackMenuPage", GetMenu(), true);
	CGameUILayer@ layer = GetTitlepackMenuLayer();
	layer.IsVisible = false;
	layer.ManialinkPage = "";
}

void GoToMainMenu() {
	MLHook::RemoveAllInjectedML();
	MLHook::InjectManialinkToMenu("TitlepackMenuPage", GetMenu(), true);
}

void GoToLocalPlay() {
	MLHook::RemoveAllInjectedML();
	MLHook::InjectManialinkToMenu("MapBrowserMenuPage", GetMapBrowserUI(), true);
}

void GoToReplays() {
	MLHook::RemoveAllInjectedML();
	MLHook::InjectManialinkToMenu("ReplayBrowserMenuPage", GetReplayBrowserUI(), true);
}

void OnDestroyed() {
	auto app = cast<CGameManiaPlanet>(GetApp());
	app.LoadedManiaTitle.IdName = "Trackmania"; 
	MLHook::RemoveAllInjectedML(); 
	RestoreNormalMenu();
	GetTitlepackMenuLayer().ManialinkPage = "";
}
void OnDisabled() { OnDestroyed(); }

void Main()
{
	auto @thehook = MyHookClass();
	MLHook::RequireVersionApi("0.4.0");
	MLHook::RegisterMLHook(thehook, "MLHook_menu_action", true);
	MLHook::InjectManialinkToMenu("EnterTitlepackPage", GetEnterTitlepackButton(), true); 
	ShowCheckForUpdateUI();

#if TMNEXT
	canUseLocalPlay = Permissions::PlayLocalMap()
	               && Permissions::CreateLocalServer()
	               && Permissions::FindLocalServer();
	canUseEditors = Permissions::OpenAdvancedMapEditor()
	             && Permissions::CreateAndUploadMap()
	             && Permissions::CreateLocalMap()
	             && Permissions::PlayAgainstReplay()
	             && Permissions::OpenSkinEditor();
#endif

	auto app = cast<CGameManiaPlanet>(GetApp());	
		
	while (true) {
		g_inCustomMenu = false;
		for (uint i = 0; i < app.ActiveMenus.Length; i++) {
			if (app.ActiveMenus[i].CurrentFrame.IdName == "FrameMenuCustom") {
				g_inCustomMenu = true;
			}
		}

		// Update TP (but not when downloading maps separetely)
		if(updateRequested && !downloadAllMapsRequested) {
			isUpdating = true;
			updateRequested = false;

			CGameUILayer@ layer = GetTitlepackMenuLayer();
			layer.IsVisible = true;
			TitlepackUpdater::Update(layer);
			layer.ManialinkPage = "";
			layer.IsVisible = false;
			RestoreNormalMenu();
			isUpdating = false;
		}

		// Download all maps (but not when updating TP)
		if(downloadAllMapsRequested && !updateRequested) {
			isUpdating = true;
			downloadAllMapsRequested = false;

			CGameUILayer@ layer = GetTitlepackMenuLayer();
			layer.IsVisible = true;
			TitlepackUpdater::DownloadAllMaps(layer);
			layer.ManialinkPage = "";
			layer.IsVisible = false;
			RestoreNormalMenu();
			Game_RefreshMapList();
			isUpdating = false;
		}


		// Enter platform titlepack; async
		if(enabledRequested && !isEnabled)
		{	
			isEnabled = true;
			enabledRequested = false;
			
			if(!TitlepackUpdater::HasMusic()) {
				CGameUILayer@ layer = GetTitlepackMenuLayer();
				layer.ManialinkPage = LoadscreenText("Unable to load titlepack - missing files!\nPlease update titlepack's files! [use OP menu / F3]");
				sleep(3000);
				layer.ManialinkPage = "";
				LeaveTitlepack();
				continue;
			}
							
			ClearAllMenuLayers();
			//auto loadscreen = GetMenuCustom().UILayers[3];
			auto defaultMenuLayer = GetDefaultMenuLayer();
			auto menuBgLayer = GetMenuBg();
			if(defaultMenuLayer != null)
				defaultMenuLayer.IsVisible = false;
			CGameUILayer@ layer = GetTitlepackMenuLayer();			
			layer.ManialinkPage = LoadscreenText("Please wait...");
			layer.InAnimation = CGameUILayer::EUILayerAnimation::None;
			layer.IsVisible = true;
			menuBgLayer.ManialinkPage = GetTesterCheckUI();
			menuBgLayer.IsVisible = true;
			sleep(2000);
			menuBgLayer.ManialinkPage = GetMenuBackground();
			sleep(100);
			layer.IsVisible = false;
			layer.InAnimation = CGameUILayer::EUILayerAnimation::LeftFast;
			/*loadscreen.ManialinkPage = GetMenuLoadscreen();
			loadscreen.IsVisible = true;
			loadscreen.Type = CGameUILayer::EUILayerType::LoadingScreen;*/

			MLHook::InjectManialinkToMenu("TitlepackMenuPage", GetMenu(), true); 

			if(!TitlepackUpdater::IsUpToDate()) {
				ShowUpdateDialog();
			}
		}

		
		if(disabledRequested && isEnabled)	{
			auto menuBgLayer = GetMenuBg();
			menuBgLayer.ManialinkPage = "";
			isEnabled = false;
			disabledRequested = false;
			ShowLeaveTitlepackDialog();
			RestoreNormalMenu();
			if(GetDefaultMenuLayer() != null)	
				GetDefaultMenuLayer().IsVisible = true;
			MLHook::InjectManialinkToMenu("EnterTitlepackPage", GetEnterTitlepackButton(), true); 
		}

		yield();
	}
}

CTrackMania@ GetTmApp() {
	return cast<CTrackMania>(GetApp());
}

CGameManiaAppTitle@ GetMenuCustom() {
	return GetTmApp().MenuManager.MenuCustom_CurrentManiaApp;
}

CGameUILayer@ GetMenuBg()	
{
	return GetMenuCustom().UILayers[9];
}

CGameUILayer@ GetDefaultMenuLayer()	
{
	auto ls = GetMenuCustom().UILayers;
    for (uint i = 0; i < ls.Length; i++) {
        auto layer = ls[i];
		if(layer == null)
			continue;
        if (layer.IsVisible) {
            if (layer.ManialinkPageUtf8.StartsWith("\n<manialink name=\"Page_")) {
				return layer;
            }
        }
    }
	return null;
}

void ClearAllMenuLayers()
{
	removedLayerIds.RemoveRange(0, removedLayerIds.Length - 1);
	auto ls = GetMenuCustom().UILayers;
    for (uint i = 0; i < ls.Length; i++) {
        auto layer = ls[i];
		if(layer == null)
			continue;
        if (layer.IsVisible) {
            layer.IsVisible = false;
			removedLayerIds.InsertLast(i);
        }
    }
}


CGameUILayer@ GetTitlepackMenuLayer()	
{
	return GetMenuCustom().UILayers[11];
}

void ShowLeaveTitlepackDialog() {
	auto layer = GetTitlepackMenuLayer();
	layer.IsVisible = true;
	layer.ManialinkPage = LoadscreenText("Please wait...");
	layer.InAnimation = CGameUILayer::EUILayerAnimation::None;
	sleep(500);
	layer.ManialinkPage="";
}

void RestoreNormalMenu()
{
	auto ls = GetMenuCustom().UILayers;
	for(int i = 0; i< removedLayerIds.Length; i++) {
		auto layer = ls[removedLayerIds[i]];
		if(layer == null)
			continue;
		layer.IsVisible = true;	
	}
	removedLayerIds.RemoveRange(0, removedLayerIds.Length - 1);
}