namespace TitlepackUpdater{

	CGameUILayer@ loadingLayer;
	string scriptsRepositoryName = "https://raw.githubusercontent.com/ArkadySK/TMNextPlatform";
	string dataRepositoryName = "https://raw.githubusercontent.com/ArkadySK/TMNextPlatformTitlepack";
	string branch = "main";
	string ScriptVersion;

	bool IsUpToDate() {
		if(!HasMusic())
			return false;

		auto@ netScriptVersion = Net::HttpGet(dataRepositoryName + "/" + branch + "/title_version.json");
		while (!netScriptVersion.Finished()) {
			yield();
		};
		ScriptVersion = netScriptVersion.String();

		Tools::CreateUserDataFolderIfNotPresent("TMNextPlatform");
		string localVersionJSONPath = IO::FromUserGameFolder("TMNextPlatform\\title_version.json"); 
		localVersionJSONPath = localVersionJSONPath.Replace("\\", "/");

		string localVersion = Json::Write(Json::FromFile(localVersionJSONPath)).Replace("\"", "");
		ScriptVersion = ScriptVersion.Replace("\"", "");
		localVersion = localVersion.Replace("\\", "");

		print("Newest version: " + ScriptVersion);
		print("Version on your drive: " + localVersion);
		return ScriptVersion == localVersion;
	}

	bool HasMusic()	{
		string menuMusicFilePath = IO::FromUserGameFolder("Media\\Musics\\TMNextPlatform\\BlurredMemories.ogg");
    	return IO::FileExists(menuMusicFilePath);
	}

	void Update(CGameUILayer@ loadingLayerFromMenu)
	{
		@loadingLayer = @loadingLayerFromMenu;
		loadingLayer.ManialinkPage = LoadscreenText("Please wait...");
		
		DownloadScripts();
		try	{
			DownloadCoreFiles();
			sleep(1000);
			GetMenuCustom().Dialog_Message("Successfully updated to version: " + ScriptVersion + "\nPlease restart your game to apply changes.");
		}
		catch	{
			GetMenuCustom().Dialog_Message("Download failed. Please report this issue.");
			return;
		}

		Tools::CreateUserDataFolderIfNotPresent("TMNextPlatform");
		string currentVersionJSON = IO::FromUserGameFolder("TMNextPlatform\\title_version.json"); 
		Json::ToFile(currentVersionJSON, ScriptVersion);

		UI::ShowNotification("\\$0d0" + Icons::Kenney::Flag + "Platform titlepack has been updated.", "\\$fffRunning version " + ScriptVersion + ".", 10000);	
	}


	void DownloadCoreFiles() {
		UI::ShowNotification("\\$0cf" + Icons::Download + "Redownloading Platform titlepack...", "\\$fffDowloading all files...", 10000);
		
		// Media/Images/Icon
		auto@ netMediaImagesTMNextPlatformIcon_discord = Net::HttpGet(dataRepositoryName + "/" + branch + "/Media/Images/TMNextPlatform/Icon/discord.png");
		auto@ netMediaImagesTMNextPlatformIcon_Github = Net::HttpGet(dataRepositoryName + "/" + branch + "/Media/Images/TMNextPlatform/Icon/Github.png");
		
		// Media/Images
		auto@ netMediaImagesTMNextPlatform_Loadscreen = Net::HttpGet(dataRepositoryName + "/" + branch + "/Media/Images/TMNextPlatform/Loadscreen.jpg");
		auto@ netMediaImagesTMNextPlatformBackground_MenuBg = Net::HttpGet(dataRepositoryName + "/" + branch + "/Media/Images/TMNextPlatform/Background/MenuBg.webm");
		auto@ netMediaImagesTMNextPlatform_PlatformBackground = Net::HttpGet(dataRepositoryName + "/" + branch + "/Media/Images/TMNextPlatform/PlatformBackground.jpg");
		
		// Media/Sounds
		auto@ netMediaSoundsTMNextPlatform_Empty = Net::HttpGet(dataRepositoryName + "/" + branch + "/Media/Sounds/TMNextPlatform/empty.wav");

		loadingLayer.ManialinkPage = LoadscreenText("Updating data...\nDownloading media...");
		while (!netMediaImagesTMNextPlatformIcon_discord.Finished() || 
				!netMediaImagesTMNextPlatformIcon_Github.Finished() || 

				!netMediaImagesTMNextPlatform_Loadscreen.Finished() || 
				!netMediaImagesTMNextPlatformBackground_MenuBg.Finished() || 
				!netMediaImagesTMNextPlatform_PlatformBackground.Finished() ||  
				
				!netMediaSoundsTMNextPlatform_Empty.Finished()){
			yield();
		}

		// Media/Musics
		auto@ netMediaMusicsTMNextPlatform_BlurredMemories = Net::HttpGet(dataRepositoryName + "/" + branch + "/Media/Musics/TMNextPlatform/BlurredMemories.ogg");
		auto@ netMediaMusicsTMNextPlatform_DeveloperNightClub = Net::HttpGet(dataRepositoryName + "/" + branch + "/Media/Musics/TMNextPlatform/DeveloperNightClub.ogg");
		auto@ netMediaMusicsTMNextPlatform_Heartbeat = Net::HttpGet(dataRepositoryName + "/" + branch + "/Media/Musics/TMNextPlatform/Heartbeat.ogg");
		
		auto@ netMediaMusicsCanyonPlatformRace_BuffaloSkull = Net::HttpGet(dataRepositoryName + "/" + branch + "/Media/Musics/CanyonPlatform/Race/Buffalo%20Skull.ogg");
		auto@ netMediaMusicsCanyonPlatformRace_FasterThanLight = Net::HttpGet(dataRepositoryName + "/" + branch + "/Media/Musics/CanyonPlatform/Race/Faster%20Than%20Light.ogg");
		auto@ netMediaMusicsCanyonPlatformRace_SandStorms = Net::HttpGet(dataRepositoryName + "/" + branch + "/Media/Musics/CanyonPlatform/Race/Sand%20Storms.ogg");
		auto@ netMediaMusicsCanyonPlatformRace_TheLostOasis = Net::HttpGet(dataRepositoryName + "/" + branch + "/Media/Musics/CanyonPlatform/Race/The%20Lost%20Oasis.ogg");
		
		loadingLayer.ManialinkPage = LoadscreenText("Updating data...\nDownloading musics...");
		while (!netMediaMusicsTMNextPlatform_BlurredMemories.Finished() || 
				!netMediaMusicsTMNextPlatform_DeveloperNightClub.Finished() || 
				!netMediaMusicsTMNextPlatform_Heartbeat.Finished() ||
				
				!netMediaMusicsCanyonPlatformRace_BuffaloSkull.Finished() || 
				!netMediaMusicsCanyonPlatformRace_FasterThanLight.Finished() || 
				!netMediaMusicsCanyonPlatformRace_SandStorms.Finished() || 
				!netMediaMusicsCanyonPlatformRace_TheLostOasis.Finished()){
			yield();
		}
    
		// Maps
		auto@ netMapsTMNextPlatform0_PlatformDemo01 = Net::HttpGet(dataRepositoryName + "/" + branch + "/Maps/TMNextPlatform/0/PlatformDemo01.Map.Gbx");
		auto@ netMapsTMNextPlatform0_PlatformDemo02 = Net::HttpGet(dataRepositoryName + "/" + branch + "/Maps/TMNextPlatform/0/PlatformDemo02.Map.Gbx");
		auto@ netMapsTMNextPlatform0_PlatformDemo03 = Net::HttpGet(dataRepositoryName + "/" + branch + "/Maps/TMNextPlatform/0/PlatformDemo03.Map.Gbx");
		auto@ netMapsTMNextPlatform0_PlatformDemo04 = Net::HttpGet(dataRepositoryName + "/" + branch + "/Maps/TMNextPlatform/0/PlatformDemo04.Map.Gbx");
		auto@ netMapsTMNextPlatform0_PlatformDemo05 = Net::HttpGet(dataRepositoryName + "/" + branch + "/Maps/TMNextPlatform/0/PlatformDemo05.Map.Gbx");
		
		loadingLayer.ManialinkPage = LoadscreenText("Updating data...\nDownloading campaign maps...");
		while (	!netMapsTMNextPlatform0_PlatformDemo01.Finished() || 
				!netMapsTMNextPlatform0_PlatformDemo02.Finished() || 
				!netMapsTMNextPlatform0_PlatformDemo03.Finished() || 
				!netMapsTMNextPlatform0_PlatformDemo04.Finished() || 
				!netMapsTMNextPlatform0_PlatformDemo05.Finished() ){
			yield();
		}

		array<string> folders = {
			"/Media",
			"/Media/Images",
			"/Media/Images/TMNextPlatform",
			"/Media/Images/TMNextPlatform/Icon",
			"/Media/Images/TMNextPlatform/Background",
			"/Media/Sounds",
			"/Media/Sounds/TMNextPlatform",
			"/Media/Musics",
			"/Media/Musics/TMNextPlatform",
			"/Media/Musics/CanyonPlatform",
			"/Media/Musics/CanyonPlatform/Race",
			"/Maps",
			"/Maps/TMNextPlatform",
			"/Maps/TMNextPlatform/0"
		};
		for(int i = 0; i < folders.Length; i++) {
			Tools::CreateUserDataFolderIfNotPresent(folders[i]);
		}

		IO::File fileMediaImagesTMNextPlatformIcon_discord(IO::FromUserGameFolder("/Media/Images/TMNextPlatform/Icon/discord.png"), IO::FileMode::Write);
			fileMediaImagesTMNextPlatformIcon_discord.Write(netMediaImagesTMNextPlatformIcon_discord.String());
		IO::File fileMediaImagesTMNextPlatformIcon_Github(IO::FromUserGameFolder("/Media/Images/TMNextPlatform/Icon/Github.png"), IO::FileMode::Write);
			fileMediaImagesTMNextPlatformIcon_Github.Write(netMediaImagesTMNextPlatformIcon_Github.String());

		IO::File fileMediaImagesTMNextPlatform_Loadscreen(IO::FromUserGameFolder("/Media/Images/TMNextPlatform/Loadscreen.jpg"), IO::FileMode::Write);
			fileMediaImagesTMNextPlatform_Loadscreen.Write(netMediaImagesTMNextPlatform_Loadscreen.String());		
		try{
			IO::File fileMediaImagesTMNextPlatformBackground_MenuBg(IO::FromUserGameFolder("/Media/Images/TMNextPlatform/Background/MenuBg.webm"), IO::FileMode::Write);
				fileMediaImagesTMNextPlatformBackground_MenuBg.Write(netMediaImagesTMNextPlatformBackground_MenuBg.String());
		}
		catch {
			print("Error when saving Background/MenuBg.webm file. The titlepack will work anyway.");
		}
		IO::File fileMediaImagesTMNextPlatform_PlatformBackground(IO::FromUserGameFolder("/Media/Images/TMNextPlatform/PlatformBackground.jpg"), IO::FileMode::Write);
			fileMediaImagesTMNextPlatform_PlatformBackground.Write(netMediaImagesTMNextPlatform_PlatformBackground.String());
		
		IO::File fileMediaMusicsTMNextPlatform_BlurredMemories(IO::FromUserGameFolder("/Media/Musics/TMNextPlatform/BlurredMemories.ogg"), IO::FileMode::Write);
			fileMediaMusicsTMNextPlatform_BlurredMemories.Write(netMediaMusicsTMNextPlatform_BlurredMemories.String());
		IO::File fileMediaMusicsTMNextPlatform_DeveloperNightClub(IO::FromUserGameFolder("/Media/Musics/TMNextPlatform/DeveloperNightClub.ogg"), IO::FileMode::Write);
			fileMediaMusicsTMNextPlatform_DeveloperNightClub.Write(netMediaMusicsTMNextPlatform_DeveloperNightClub.String());
		IO::File fileMediaMusicsTMNextPlatform_Heartbeat(IO::FromUserGameFolder("/Media/Musics/TMNextPlatform/Heartbeat.ogg"), IO::FileMode::Write);
			fileMediaMusicsTMNextPlatform_Heartbeat.Write(netMediaMusicsTMNextPlatform_Heartbeat.String());

		IO::File fileMediaMusicsCanyonPlatformRace_BuffaloSkull(IO::FromUserGameFolder("/Media/Musics/CanyonPlatform/Race/Buffalo Skull.ogg"), IO::FileMode::Write);
			fileMediaMusicsCanyonPlatformRace_BuffaloSkull.Write(netMediaMusicsCanyonPlatformRace_BuffaloSkull.String());
		IO::File fileMediaMusicsCanyonPlatformRace_FasterThanLight(IO::FromUserGameFolder("/Media/Musics/CanyonPlatform/Race/Faster Than Light.ogg"), IO::FileMode::Write);
			fileMediaMusicsCanyonPlatformRace_FasterThanLight.Write(netMediaMusicsCanyonPlatformRace_FasterThanLight.String());
		IO::File fileMediaMusicsCanyonPlatformRace_SandStorms(IO::FromUserGameFolder("/Media/Musics/CanyonPlatform/Race/Sand Storms.ogg"), IO::FileMode::Write);
			fileMediaMusicsCanyonPlatformRace_SandStorms.Write(netMediaMusicsCanyonPlatformRace_SandStorms.String());
		IO::File fileMediaMusicsCanyonPlatformRace_TheLostOasis(IO::FromUserGameFolder("/Media/Musics/CanyonPlatform/Race/The Lost Oasis.ogg"), IO::FileMode::Write);
			fileMediaMusicsCanyonPlatformRace_TheLostOasis.Write(netMediaMusicsCanyonPlatformRace_TheLostOasis.String());
		
		IO::File fileMediaSoundsTMNextPlatform_Empty(IO::FromUserGameFolder("/Media/Sounds/TMNextPlatform/empty.wav"), IO::FileMode::Write);
			fileMediaSoundsTMNextPlatform_Empty.Write(netMediaSoundsTMNextPlatform_Empty.String());
		
		IO::File fileMapsTMNextPlatform0_PlatformDemo01(IO::FromUserGameFolder("/Maps/TMNextPlatform/0/PlatformDemo01.Map.Gbx"), IO::FileMode::Write);
			fileMapsTMNextPlatform0_PlatformDemo01.Write(netMapsTMNextPlatform0_PlatformDemo01.String());
		IO::File fileMapsTMNextPlatform0_PlatformDemo02(IO::FromUserGameFolder("/Maps/TMNextPlatform/0/PlatformDemo02.Map.Gbx"), IO::FileMode::Write);
			fileMapsTMNextPlatform0_PlatformDemo02.Write(netMapsTMNextPlatform0_PlatformDemo02.String());
		IO::File fileMapsTMNextPlatform0_PlatformDemo03(IO::FromUserGameFolder("/Maps/TMNextPlatform/0/PlatformDemo03.Map.Gbx"), IO::FileMode::Write);
			fileMapsTMNextPlatform0_PlatformDemo03.Write(netMapsTMNextPlatform0_PlatformDemo03.String());
		IO::File fileMapsTMNextPlatform0_PlatformDemo04(IO::FromUserGameFolder("/Maps/TMNextPlatform/0/PlatformDemo04.Map.Gbx"), IO::FileMode::Write);
			fileMapsTMNextPlatform0_PlatformDemo04.Write(netMapsTMNextPlatform0_PlatformDemo04.String());
		IO::File fileMapsTMNextPlatform0_PlatformDemo05(IO::FromUserGameFolder("/Maps/TMNextPlatform/0/PlatformDemo05.Map.Gbx"), IO::FileMode::Write);
			fileMapsTMNextPlatform0_PlatformDemo05.Write(netMapsTMNextPlatform0_PlatformDemo05.String());

	}

	void DownloadScripts() {
		UI::ShowNotification("\\$0cf" + Icons::Download + "Updating Platform titlepack...", "\\$fffDowloading update...", 3000);

		// Scripts
		auto@ netLibs_ModeManialinks = Net::HttpGet(scriptsRepositoryName + "/" + branch + "/Libs/TrackMania/ModeManialinks.Script.txt");
		auto@ netMapTypes_Platform = Net::HttpGet(scriptsRepositoryName + "/" + branch + "/MapTypes/Trackmania/Platform.Script.txt");
		auto@ netModes_LegacyMode = Net::HttpGet(scriptsRepositoryName + "/" + branch + "/Modes/TrackMania/LegacyMode.Script.txt");
		auto@ netModes_PlatformRounds = Net::HttpGet(scriptsRepositoryName + "/" + branch + "/Modes/TrackMania/PlatformRounds.Script.txt");
		
		loadingLayer.ManialinkPage = LoadscreenText("Downloading scripts and menus...");
		while (!netLibs_ModeManialinks.Finished() || 
				!netMapTypes_Platform.Finished() || 
				!netModes_LegacyMode.Finished() || 
				!netModes_PlatformRounds.Finished()){
			yield();
		}

		array<string> folders = {
			"/Scripts",
			"/Scripts/Libs",
			"/Scripts/Libs/Trackmania",
			"/Scripts/Modes",
			"/Scripts/Modes/Trackmania",
			"/Scripts/MapTypes",
			"/Scripts/MapTypes/Trackmania"
		};
		for(int i = 0; i < folders.Length; i++) {
			Tools::CreateUserDataFolderIfNotPresent(folders[i]);
		}

		IO::File fileLibsModeManialinks(IO::FromUserGameFolder("/Scripts/Libs/TrackMania/ModeManialinks.Script.txt"), IO::FileMode::Write);
		fileLibsModeManialinks.Write(netLibs_ModeManialinks.String());
		IO::File fileMapTypesPlatform(IO::FromUserGameFolder("/Scripts/MapTypes/Trackmania/Platform.Script.txt"), IO::FileMode::Write);
		fileMapTypesPlatform.Write(netMapTypes_Platform.String());
		IO::File fileModesLegacyMode(IO::FromUserGameFolder("/Scripts/Modes/TrackMania/LegacyMode.Script.txt"), IO::FileMode::Write);
		fileModesLegacyMode.Write(netModes_LegacyMode.String());
		IO::File fileModesPlatformRounds(IO::FromUserGameFolder("/Scripts/Modes/TrackMania/PlatformRounds.Script.txt"), IO::FileMode::Write);
		fileModesPlatformRounds.Write(netModes_PlatformRounds.String());
	}

	// This method is usable only for testers, it is called from a secret tester only menus...
	void DownloadAllMaps(CGameUILayer@ loadingLayer) {
		string[] folders = {
			"/Maps/TMNextPlatform_Campaign",
			"/Maps/TMNextPlatform_Campaign/0",
			"/Maps/TMNextPlatform_Campaign/1",
			"/Maps/TMNextPlatform_Campaign/2",
			"/Maps/TMNextPlatform_Campaign/3",
			"/Maps/TMNextPlatform_Campaign/4"
		};
		for(int i = 0; i < folders.Length; i++) {
			Tools::CreateUserDataFolderIfNotPresent(folders[i]);
		}

		string[] colors = {"A", "B", "C", "D", "E"};
		for(int colorNb = 0; colorNb < 5; colorNb++) {
			for(int mapNb = 1; mapNb<= 10; mapNb++) {
				string mapNbString = "" + mapNb;
				if(mapNb < 10)
					mapNbString = "0" + mapNb;
				string mapName = "Platform" + colors[colorNb] + mapNbString;
				string mapPath = "/Maps/TMNextPlatform_Campaign/" + colorNb + "/" + mapName + ".Map.Gbx";
				try{
					auto@ netMap = Net::HttpGet(dataRepositoryName + "/" + branch + mapPath);
					while (!netMap.Finished()) {yield();}
					loadingLayer.ManialinkPage = LoadscreenText("Downloading map: " + mapName + ".Map.Gbx");
					IO::File mapFile(IO::FromUserGameFolder(mapPath), IO::FileMode::Write);
					mapFile.Write(netMap.String());
				}
				catch {
					print("ERROR: Saving or downloading of '" + mapName + "'' was not successful.");
				}
			}
		}
	}
}
